# Installation
bootstrap_storybook theme uses [Webpack](https://webpack.js.org) to compile and bundle SASS and JS.
It is a component-first designed theme, so each component is available for Drupal and Storybook

#### Step 1
Make sure you have Node and npm installed.
You can read a guide on how to install node here: https://docs.npmjs.com/getting-started/installing-node

If you prefer to use [Yarn](https://yarnpkg.com) instead of npm, install Yarn by following the guide [here](https://yarnpkg.com/docs/install).

#### Step 2
Go to the root of your SUBTHEME and run the following commands: `npm install` or preferrably `yarn install`.
when you're done make sure to update any security issues for node packages if there's any: `npm audit fix`

#### Step 3
Copy the `config.example.js` to `config.js` in the config directory of your subtheme: `cp config/config.example.js config/config.js`
Update **proxy** variable to your localhost address in `config/config.js`


#### Step 4
Make sure to build the theme first: `npm run crazy` or `yarn crazy`

#### Step 5
Make sure that you have the `components` module downloaded and enabled: `composer require 'drupal/components:^2.0'` and then enable the module: `drush en components -y`

### Step 6
Enable and set the active theme to your newly created SUBTHEMENAME: `drush then SUBTHEMENAME -y; drush config-set system.theme default SUBTHEMENAME -y`

# Critical CSS
So currently for building assets we are using [laravel-mix](https://laravel-mix.com/), it's a tool to automate the builds and run commands defined in `package.json`

### How to use critical CSS
For running production builds and creating the Critical CSS file: `npm run production`

Note that the critical-css file is only being generated by running the production command, it will build and save the asset into the `assets/css/critical/PATH_critical.min.css`, we currently only use the front path, so it's named: `front_critical.min.css` which should get defined via a library in `SUBTHEMENAME.libraries.yml` and loaded in a twig file afterwards, in case you need specific path-based critical css files besides your front, you may add more paths to `webpack.mix.js` in the `urls` array section, f.x:
```
  urls: [
    { url: '/', template: 'front' },
    { url: '/blog', template: 'blog' },
  ],
```

# Updating the subtheme
In some scenarios you might like to update your subtheme with a new release of bootstrap_storybook, for those cases you may follow the steps below:

- Update the parent theme: `composer update drupal/bootstrap_storybook`
- Make sure the bootstrap_storybook is enabled: `drush en bootstrap_storybook`
- Set default theme to something else so we can temporarily uninstall it and move it
```
drush config-set system.theme default bartik
drush theme:uninstall SUBTHEMENAME
mv web/themes/custom/SUBTHEMENAME ../SUBTHEMENAME.bak
drush --include="web/themes/contrib/bootstrap_storybook" bootstrap_storybook:create "My Bootstrap Storybook Theme"
```
- Have a look at the differences, and make adjustments as necessary
`git diff -- . ':(exclude)web/themes/custom/my_bootstrap_storybook/package-lock.json'`
- Make sure things run smoothly
```
npm install
drush theme:enable SUBTHEMENAME
drush config-set system.theme default SUBTHEMENAME
yarn crazy
```
